//+------------------------------------------------------------------+
//|                                           ATM RSI Zone Trade.mq4 |
//|                                        Copyright 2016, ATM Corp. |
//|                                               http://www.ATM.net |
//+------------------------------------------------------------------+
#property copyright "Copyright 2016, ATM Corp."
#property link      "http://www.ATM.net"

#property indicator_chart_window
#property indicator_buffers 8

#property indicator_color1 clrGray
#property indicator_color2 clrGray
#property indicator_color3 clrGray
#property indicator_color4 clrGray
#property indicator_color5 clrGray
#property indicator_color6 clrGray
#property indicator_color7 clrGray
#property indicator_color8 clrGray
#property indicator_width1 1
#property indicator_width2 1
#property indicator_width3 3
#property indicator_width4 3
#property indicator_width5 1
#property indicator_width6 1
#property indicator_width7 3
#property indicator_width8 3

enum e_method{ SMA=MODE_SMA, EMA=MODE_EMA, SMMA=MODE_SMMA, LWMA=MODE_LWMA };

extern ENUM_TIMEFRAMES TimeFrame=PERIOD_CURRENT;
extern int      RSI_Period       = 10;
extern int      RSI_MA_Period    = 10;
extern e_method MA_Method        = LWMA;
extern string   Comment0         = "- Inverse RSI/MA-of-RSI Crossing -";
extern bool     Inverse_Crossing = true;

double Up_High[];
double Up_Low[];
double Up_Open[];
double Up_Close[];
double Dn_High[];
double Dn_Low[];
double Dn_Open[];
double Dn_Close[];

double RSI[];
double MA[];

int Bullish = 1;
int Bearish = -1;
int Neutral = 0;
string indicatorName;

int init(){
 
   IndicatorShortName("RSI Zone Trade");
   IndicatorBuffers(10);   
   SetIndexBuffer(0,Up_High);
   SetIndexBuffer(1,Up_Low);
   SetIndexBuffer(2,Up_Open);
   SetIndexBuffer(3,Up_Close);
   SetIndexBuffer(4,Dn_High);
   SetIndexBuffer(5,Dn_Low);
   SetIndexBuffer(6,Dn_Open);
   SetIndexBuffer(7,Dn_Close);
   SetIndexBuffer(8,RSI);
   SetIndexBuffer(9,MA);
   
   for (int i=0; i<8; i++){
      SetIndexStyle(i,DRAW_HISTOGRAM);
   }

   TimeFrame=MathMax(TimeFrame,_Period);
   indicatorName=WindowExpertName();
   return(0);
   
}

int start(){   

   int i, bias;
   int counted_bars=IndicatorCounted();
   int limit = Bars-counted_bars-1;
      if (TimeFrame!=_Period)
      {
         limit = MathMin(Bars-1,MathMax(2*TimeFrame/_Period,limit));
         for(i=limit; i>=0; i--)
         {
            int y=iBarShift(NULL,TimeFrame,Time[i]);
            Up_High[i]  = iCustom(NULL,TimeFrame,indicatorName,0,RSI_Period,RSI_MA_Period,MA_Method,"",Inverse_Crossing,0,y);
            Up_Low[i]   = iCustom(NULL,TimeFrame,indicatorName,0,RSI_Period,RSI_MA_Period,MA_Method,"",Inverse_Crossing,1,y);
            Up_Open[i]  = iCustom(NULL,TimeFrame,indicatorName,0,RSI_Period,RSI_MA_Period,MA_Method,"",Inverse_Crossing,2,y);
            Up_Close[i] = iCustom(NULL,TimeFrame,indicatorName,0,RSI_Period,RSI_MA_Period,MA_Method,"",Inverse_Crossing,3,y);
            Dn_High[i]  = iCustom(NULL,TimeFrame,indicatorName,0,RSI_Period,RSI_MA_Period,MA_Method,"",Inverse_Crossing,4,y);
            Dn_Low[i]   = iCustom(NULL,TimeFrame,indicatorName,0,RSI_Period,RSI_MA_Period,MA_Method,"",Inverse_Crossing,5,y);
            Dn_Open[i]  = iCustom(NULL,TimeFrame,indicatorName,0,RSI_Period,RSI_MA_Period,MA_Method,"",Inverse_Crossing,6,y);
            Dn_Close[i] = iCustom(NULL,TimeFrame,indicatorName,0,RSI_Period,RSI_MA_Period,MA_Method,"",Inverse_Crossing,7,y);
         }
         return(0);
      }
 
   for(i=limit; i>=0; i--){
   
      Up_High[i]  = EMPTY_VALUE;
      Up_Low[i]   = EMPTY_VALUE;
      Up_Open[i]  = EMPTY_VALUE;
      Up_Close[i] = EMPTY_VALUE;
      Dn_High[i]  = EMPTY_VALUE;
      Dn_Low[i]   = EMPTY_VALUE;
      Dn_Open[i]  = EMPTY_VALUE;
      Dn_Close[i] = EMPTY_VALUE;
   
      RSI[i] = iRSI(NULL,0,RSI_Period,PRICE_CLOSE,i);
      MA[i]  = iMAOnArray(RSI,0,RSI_MA_Period,0,ENUM_MA_METHOD(MA_Method),i);
      
      bias=ColorCandle(RSI[i], MA[i]);
   
      if(bias>0){
         
         Up_High[i]  = High[i];
         Up_Low[i]   = Low[i];  
         Up_Open[i]  = MathMin(Open[i],Close[i]);
         Up_Close[i] = MathMax(Open[i],Close[i])+Point;  
      
      }
      else if(bias<0){
         
         Dn_High[i]  = High[i];
         Dn_Low[i]   = Low[i];  
         Dn_Open[i]  = MathMin(Open[i],Close[i]);
         Dn_Close[i] = MathMax(Open[i],Close[i])+Point;
      
      }

   }
 
   return(0);

}

int ColorCandle(double rsi_value, double ma_value){
 
 if (rsi_value > 50 && ((Inverse_Crossing==false && rsi_value > ma_value) || (Inverse_Crossing && rsi_value < ma_value)))
   
   return (Bullish);
 
 else if (rsi_value < 50 && ((Inverse_Crossing==false && rsi_value < ma_value) || (Inverse_Crossing && rsi_value > ma_value)))
   
   return (Bearish);
 
 else
   
   return (Neutral);

}