//+------------------------------------------------------------------+
//|                                       Candle_Size_Alert_v3_2.mq4 |
//|                                                                  |
//|                                                                  |
//+------------------------------------------------------------------+
#property copyright ""
#property link      "None"
#property version   "3.20"
#property strict
#property indicator_chart_window
//--- input parameters
input int 	CandlePointM1 = 40;			// Minimum candle points for Period M1
input int 	CandlePointM5 = 100;			// Minimum candle points for Period M5
input int 	CandlePointH1 = 500;			// Minimum candle points for Period H1

int 			timeFrame[] 	 = {PERIOD_M1, PERIOD_M5, PERIOD_H1};
string 		currencyPairs[] = {"AUDCAD", "AUDNZD", "AUDJPY", "AUDUSD"};
datetime 	alert_bar[3][3];
datetime 	candleTime[3][3];
int 			TFNO; 				//No of TimeFrame
int 			CPNO; 				//No of Currency pairs
string 		sufix;	 			//Additional letters of Currency pair
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int OnInit()
{
	//--- indicator buffers mapping
	TFNO = ArraySize(timeFrame);
	CPNO = ArraySize(currencyPairs);
	
	if (StringLen(Symbol()) > 6)
	{
		sufix = StringSubstr(Symbol(), 6, 0);
		for (int i = 0; i < CPNO; i++)
			currencyPairs[i] += sufix;
	}
	//---
	return(INIT_SUCCEEDED);
}
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime &time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[])
{
	//---
	int 					range;
	double 				candleLow, candleHigh;
	
	ArrayResize(candleTime, CPNO);
	ArrayResize(alert_bar,  CPNO);
	ArrayInitialize(alert_bar, 0);
	
	for (int i = 0; i < CPNO; i++) //currencyPairs
	{
		for (int j = 0 ; j < TFNO ; j++) //timeFrame
		{
			candleLow  = iLow(currencyPairs[i],  timeFrame[j], 0);
			candleHigh = iHigh(currencyPairs[i], timeFrame[j], 0);
			range      = int((candleHigh - candleLow) / _Point);

			candleTime[i][j] = iTime(currencyPairs[i], timeFrame[j], 0);
			
			if (alert_bar[i][j] != candleTime[i][j])
			{
				if (timeFrame[j] == PERIOD_M1 && range > CandlePointM1)
				{
					Alert(currencyPairs[i], " PERIOD_M1", " candle > ", CandlePointM1, " points");
					alert_bar[i][j] = candleTime[i][j];
				}
				
				if (timeFrame[j] == PERIOD_M5 && range > CandlePointM5)
				{
					Alert(currencyPairs[i], " PERIOD_M5", " candle > ", CandlePointM5, " points");
					alert_bar[i][j] = candleTime[i][j];
				}
				
				if (timeFrame[j] == PERIOD_H1 && range > CandlePointH1)
				{
					Alert(currencyPairs[i], " PERIOD_H1", " candle > ", CandlePointH1, " points");
					alert_bar[i][j] = candleTime[i][j];
				}
			}
		}
	}
	//--- return value of prev_calculated for next call
	return(rates_total);
}
//+------------------------------------------------------------------+
