//------------------------------------------------------------------
//   AdaptiveLaguerreFilter.mq4
#property copyright "mladen"
#property link      "www.forex-tsd.com"
//------------------------------------------------------------------
#property indicator_chart_window
#property indicator_buffers 3
#property indicator_color1  LimeGreen
#property indicator_color2  Orange
#property indicator_color3  Orange
#property indicator_width1  2
#property indicator_width2  2
#property indicator_width3  2

//
//
//
//
//

extern string TimeFrame   = "current time frame";
extern int    LookBack    = 20;
extern int    Median      = 5;
extern int    Length      = 7;
extern int    PriceType   = PRICE_MEDIAN;

//
//
//
//
//

double Filter[];
double Filterda[];
double Filterdb[];
double L0[];
double L1[];
double L2[];
double L3[];
double sortDiff[];
double alphas[];
double slope[];

//
//
//
//
//

int    timeFrame;
string IndicatorFileName;
bool   ReturningBars;
bool   CalculatingAlf;

//+------------------------------------------------------------------+
//|
//+------------------------------------------------------------------+
//
//
//
//

int init() 
{

   IndicatorBuffers(8);
   SetIndexBuffer(0,Filter);   SetIndexLabel(0,"ALF");
   SetIndexBuffer(1,Filterda); SetIndexLabel(1,"ALF");
   SetIndexBuffer(2,Filterdb); SetIndexLabel(2,"ALF");
   SetIndexBuffer(3,L0);
   SetIndexBuffer(4,L1);
   SetIndexBuffer(5,L2);
   SetIndexBuffer(6,L3);
   SetIndexBuffer(7,slope);
   ArrayResize(sortDiff,Median);

   //
   //
   //
   //
   //
         
      IndicatorFileName   = WindowExpertName();
      CalculatingAlf = (TimeFrame=="CalculateAlf"); if (CalculatingAlf) return(0);
      ReturningBars  = (TimeFrame=="returnBars");   if (ReturningBars)  return(0);
      timeFrame      = stringToTimeFrame(TimeFrame);
   
   //
   //
   //
   //
   //
   
   IndicatorShortName("Adaptive Laguerre filter "+timeFrameToString(timeFrame)+" ("+LookBack+","+Median+","+PriceType+")");
   return (0);
}
  
//+------------------------------------------------------------------+
//|
//+------------------------------------------------------------------+
//
//
//
//
//
//

double Diff[];
int start()
{
   int counted_bars = IndicatorCounted();
   int i,r,limit;
    
   if (counted_bars<0) return(-1);
   if (counted_bars>0) counted_bars--;
      limit = MathMin(Bars-counted_bars,Bars-1);
           if (ReturningBars)  { Filter[0] = limit; return(0); }

   //
   //
   //
   //
   //
   
   if (CalculatingAlf || timeFrame==Period())
   {
      if (ArraySize(Diff)!=Bars) ArrayResize(Diff,Bars);
      if (slope[limit]==-1) CleanPoint(limit,Filterda,Filterdb);
      for (i=limit, r=Bars-i-1;i>=0;i--,r++)
      {
         double price = iNoLagMa(iMA(NULL,0,1,0,MODE_SMA,PriceType,i),Length,alphas,0,i);
         Diff[r] = MathAbs(price - Filter[i+1]);

         //
         //
         //
         //
         //
      
         double HH = Diff[r];
         double LL = Diff[r];
            for (int j=0; j<LookBack; j++)
            {
               if (Diff[r-j] > HH) {HH = Diff[r-j];}
               if (Diff[r-j] < LL) {LL = Diff[r-j];}
            }

            double alpha = 0.5;
            if (HH!=LL)
            {
               for (j=0;j<Median;j++) sortDiff[j] = (Diff[r-j]-LL)/(HH-LL);
               ArraySort(sortDiff,WHOLE_ARRAY,0,MODE_ASCEND);
         
               if (MathMod(Median,2.0) != 0)
                     alpha = sortDiff[Median/2];         
               else  alpha = (sortDiff[Median/2]+sortDiff[(Median/2)-1])/2;
            }
      
         //
         //
         //
         //
         //
      
         L0[i] = alpha*price + (1 - alpha)*L0[i+1];
         L1[i] = -(1 - alpha)*L0[i] + L0[i+1] + (1 - alpha)*L1[i+1];
         L2[i] = -(1 - alpha)*L1[i] + L1[i+1] + (1 - alpha)*L2[i+1];
         L3[i] = -(1 - alpha)*L2[i] + L2[i+1] + (1 - alpha)*L3[i+1];
         Filter[i]   = (L0[i] + 2.0 * L1[i] + 2.0 * L2[i] + L3[i]) / 6.0;
         Filterda[i] = EMPTY_VALUE;
         Filterdb[i] = EMPTY_VALUE;
         slope[i]    = slope[i+1];
            if (Filter[i]>Filter[i+1]) slope[i] =  1;
            if (Filter[i]<Filter[i+1]) slope[i] = -1;
            if (slope[i]==-1) PlotPoint(i,Filterda,Filterdb,Filter);
      }
      return(0);
   }  
   
   //
   //
   //
   //
   //
   
   if (timeFrame > Period()) limit = MathMax(limit,MathMin(Bars,iCustom(NULL,timeFrame,IndicatorFileName,"returnBars",0,0)*timeFrame/Period()));
   if (slope[limit]==-1) CleanPoint(limit,Filterda,Filterdb);
   for (i=limit;i>=0;i--)
   {
      int y = iBarShift(NULL,timeFrame,Time[i]);
         Filter[i]   = iCustom(NULL,timeFrame,IndicatorFileName,"CalculateAlfr",LookBack,Median,Length,PriceType,0,y);
         slope[i]    = iCustom(NULL,timeFrame,IndicatorFileName,"CalculateAlfr",LookBack,Median,Length,PriceType,7,y);
         Filterda[i] = EMPTY_VALUE;
         Filterdb[i] = EMPTY_VALUE;
            if (slope[i]==-1) PlotPoint(i,Filterda,Filterdb,Filter);
   }
   return (0);   
}


//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//


double values[][3];
double prices[][1];

#define Pi       3.14159265358979323846264338327950288
#define _length  0
#define _len     1
#define _weight  2

//
//
//
//
//

double iNoLagMa(double price, int length, double &alphasArray[], int forValue, int i)
{
   if (ArrayRange(prices,0) != Bars) ArrayResize(prices,Bars);
            int r = Bars-i-1;  prices[r][forValue]=price;
   if (length<3 || r<3) return(prices[r][forValue]);
   
   //
   //
   //
   //
   //
   
   if (ArrayRange(values,0)<(forValue+1) || values[forValue][_length] != length)
   {
      double Cycle = 4.0;
      double Coeff = 3.0*Pi;
      int    Phase = length-1;
      
      if (ArrayRange(values,0)<forValue+1) ArrayResize(values,forValue+1);

         values[forValue][_length] = length;
         values[forValue][_len]    = length*4 + Phase;  
         values[forValue][_weight] = 0;  
         ArrayResize(alphasArray,values[forValue][_len]);

         for (int k=0; k<values[forValue][_len]-1; k++)
         {
            if (k<=Phase-1) 
                 double t = 1.0 * k/(Phase-1);
            else        t = 1.0 + (k-Phase+1)*(2.0*Cycle-1.0)/(Cycle*length-1.0); 
            double beta = MathCos(Pi*t);
            double g = 1.0/(Coeff*t+1); if (t <= 0.5 ) g = 1;
      
            alphasArray[k]             = g * beta;
            values[forValue][_weight] += alphasArray[k];
         }
   }
   
   //
   //
   //
   //
   //
   
   if (values[forValue][_weight]>0)
   {
      int    len = values[forValue][_len];
      double sum = 0;
           for (k=0; k < len-1; k++) sum += alphasArray[k]*prices[r-k][forValue];      
           return( sum / values[forValue][_weight]);
   }
   else return(0);           
}

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = StringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string StringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int tchar = StringGetChar(s, length);
         if((tchar > 96 && tchar < 123) || (tchar > 223 && tchar < 256))
                     s = StringSetChar(s, length, tchar - 32);
         else if(tchar > -33 && tchar < 0)
                     s = StringSetChar(s, length, tchar + 224);
   }
   return(s);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

void CleanPoint(int i,double& first[],double& second[])
{
   if ((second[i]  != EMPTY_VALUE) && (second[i+1] != EMPTY_VALUE))
        second[i+1] = EMPTY_VALUE;
   else
      if ((first[i] != EMPTY_VALUE) && (first[i+1] != EMPTY_VALUE) && (first[i+2] == EMPTY_VALUE))
          first[i+1] = EMPTY_VALUE;
}

//
//
//
//
//

void PlotPoint(int i,double& first[],double& second[],double& from[])
{
   if (first[i+1] == EMPTY_VALUE)
      {
         if (first[i+2] == EMPTY_VALUE) {
                first[i]   = from[i];
                first[i+1] = from[i+1];
                second[i]  = EMPTY_VALUE;
            }
         else {
                second[i]   =  from[i];
                second[i+1] =  from[i+1];
                first[i]    = EMPTY_VALUE;
            }
      }
   else
      {
         first[i]  = from[i];
         second[i] = EMPTY_VALUE;
      }
}