//------------------------------------------------------------------
//                           OBV Smoothed Floating levels by Investguy
//===================================================
#property copyright "Investguy"
#property link      "Investguy@gmail.com"
//------------------------------------------------------------------

#property indicator_separate_window
#property indicator_buffers 3
#property indicator_color1  LimeGreen
#property indicator_color2  PaleVioletRed
#property indicator_color3  Red
#property indicator_style1  STYLE_DOT
#property indicator_style2  STYLE_DOT
#property indicator_style3  STYLE_DOT
#property indicator_width3  1

//
//
//
//
//

extern int    LongPeriodOBV = 15;
extern double PercentOBV    = 97;
extern double SmoothLengthOBV = 5.0;
extern double SmoothPhaseOBV  = 0;
extern double StepDistanceOBV=200; //Distance between Present Value and Past Value in order to trigger the trend buffer
//
//
//Defines
#define bsmax  5
#define bsmin  6
#define volty  7
#define vsum   8
#define avolty 9
//
//
//
// Buffers
double leu[];
double led[];
double obv[];
double trend[];
// Extra Buffers
double wrk[][50];



//+------------------------------------------------------------------
//|                                                                  
//+------------------------------------------------------------------
// 
//
//
//
//

int init()
{
   IndicatorBuffers(4);
   SetIndexBuffer(0,leu);
   SetIndexBuffer(1,led);
   SetIndexBuffer(2,obv);
   SetIndexBuffer(3,trend);
   SetIndexLabel(0,"Dynamic Upper Level");
   SetIndexLabel(1,"Dynamic Lower Level");
   SetIndexLabel(2,"OBV Smoothed");
   
   IndicatorShortName("OBV Smoothed Floating Levels");
      
    
    
   return(0);
}
int deinit() { return(0); }


//+------------------------------------------------------------------
//|                                                                  
//+------------------------------------------------------------------

//
//

int start()
{
   int i,limit,counted_bars=IndicatorCounted();
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
         limit = MathMin(Bars-counted_bars,Bars-1);

   //
     
   for(i=limit; i>=0; i--) 
   {
      obv[i] =  iSmooth(iOBV(NULL,0,PRICE_CLOSE,i),SmoothLengthOBV,SmoothPhaseOBV,i);
      
               
      // Floating Levels Calculations
            double hi = obv[ArrayMaximum(obv,LongPeriodOBV,i)];
            double lo = obv[ArrayMinimum(obv,LongPeriodOBV,i)];
            double rn = hi-lo;
      leu[i] = lo+rn*PercentOBV/100.0;
      led[i] = hi-rn*PercentOBV/100.0;
       //    trend calculations
	  if       ( (obv[i] > obv[i+1])  || ((obv[i+1] < leu[i+1]) && (obv[i] > leu[i])) ) { trend[i] = 1;}
	  else if ( (obv[i] < obv[i+1]) || ((obv[i+1] > leu[i+1]) && (obv[i] < leu[i])) )  { trend[i] = -1;}
	  else {trend[i] = 0;}
      // Observe and study values to determine trend buffer change&& (StepDistanceOBV < MathAbs(obv[i] - obv[i+1])))
      //Print(" MathAbs(obv[i] - obv[i] = ", MathAbs(obv[i] - obv[i+1]) );
   }
   
   return(0);
}


// Smooth Function

double iSmooth(double price, double length, double phase, int i, int s=0)
{
   if (ArrayRange(wrk,0) != Bars) ArrayResize(wrk,Bars);
   
   int r = Bars-i-1; 
      if (r==0) { for(int k=0; k<7; k++) wrk[0][k+s]= price; for(; k<10; k++) wrk[0][k+s]= 0; return(price); }

   //
   //
   //
   //
   //
   
      double len1 = MathMax(MathLog(MathSqrt(0.5*(length-1)))/MathLog(2.0)+2.0,0);
      double pow1 = MathMax(len1-2.0,0.5);
      double del1 = price - wrk[r-1][bsmax+s];
      double del2 = price - wrk[r-1][bsmin+s];
	
         wrk[r][volty+s] = 0;
               if(MathAbs(del1) > MathAbs(del2)) wrk[r][volty+s] = MathAbs(del1); 
               if(MathAbs(del1) < MathAbs(del2)) wrk[r][volty+s] = MathAbs(del2); 
         wrk[r][vsum+s] =	wrk[r-1][vsum+s] + 0.1*(wrk[r][volty+s]-wrk[r-10][volty+s]);
   
         //
         //
         //
         //
         //
      
         double avgLen = MathMin(MathMax(4.0*length,30),150);
            if (r<avgLen)
            {
               double avg = wrk[r][vsum+s];  for (k=1; k<avgLen && (r-k)>=0 ; k++) avg += wrk[r-k][vsum+s];
                                                                                   avg /= k;
            }
            else avg = (wrk[r-1][avolty+s]*avgLen-wrk[r-toInt(avgLen)][vsum+s]+wrk[r][vsum+s])/avgLen;
            
         //
         //
         //
         //
         //
                                                           
         wrk[r][avolty+s] = avg;                                           
            if (wrk[r][avolty+s] > 0)
               double dVolty = wrk[r][volty+s]/wrk[r][avolty+s]; else dVolty = 0;   
	               if (dVolty > MathPow(len1,1.0/pow1)) dVolty = MathPow(len1,1.0/pow1);
                  if (dVolty < 1)                      dVolty = 1.0;

      //
      //
      //
      //
      //
	        
   	double pow2 = MathPow(dVolty, pow1);
      double len2 = MathSqrt(0.5*(length-1))*len1;
      double Kv   = MathPow(len2/(len2+1), MathSqrt(pow2));		
	
         if (del1 > 0) wrk[r][bsmax+s] = price; else wrk[r][bsmax+s] = price - Kv*del1;
         if (del2 < 0) wrk[r][bsmin+s] = price; else wrk[r][bsmin+s] = price - Kv*del2;

   //
   //
   //
   //
   //

      double R     = MathMax(MathMin(phase,100),-100)/100.0 + 1.5;
      double beta  = 0.45*(length-1)/(0.45*(length-1)+2);
      double alpha = MathPow(beta,pow2);

         wrk[r][0+s] = price + alpha*(wrk[r-1][0+s]-price);
         wrk[r][1+s] = (price - wrk[r][0+s])*(1-beta) + beta*wrk[r-1][1+s];
         wrk[r][2+s] = (wrk[r][0+s] + R*wrk[r][1+s]);
         wrk[r][3+s] = (wrk[r][2+s] - wrk[r-1][4+s])*MathPow((1-alpha),2) + MathPow(alpha,2)*wrk[r-1][3+s];
         wrk[r][4+s] = (wrk[r-1][4+s] + wrk[r][3+s]); 

   //
   //
   //
   //
   //

   return(wrk[r][4+s]);
}

int toInt(double value) { return(value); }   