//+------------------------------------------------------------------+
//|                                                   T3 crosses.mq4 |
//|                                                           mladen |
//|                                                                  |
//|                                                                  |
//| original T3 developed by Tim Tilson (TASC January 1998)          |
//| period lag modification by Bob Fulks and Alex Matulich 4/2003    |
//+------------------------------------------------------------------+
//mod2009fxtsd   //added  showlineson/off

#property copyright "copyleft mladen"
#property link      "mladenfx@gmail.com"

#property indicator_chart_window
#property indicator_buffers 4
#property indicator_color1  LimeGreen
#property indicator_color2  OrangeRed
#property indicator_color3  Lime
#property indicator_color4  Red

//
//
//
//
//

extern string TimeFrame      = "Current time frame";
extern int    T3FastPeriod   = 9;
extern int    T3FastPrice    = PRICE_CLOSE;
extern double T3FastHot      = 1.0;
extern bool   T3FastOriginal = false;
extern int    T3SlowPeriod   = 13;
extern int    T3SlowPrice    = PRICE_CLOSE;
extern double T3SlowHot      = 0.5;
extern bool   T3SlowOriginal = false;

extern string note            = "turn on Alert = true; turn off = false";
extern bool   alertsOn        = true;
extern bool   alertsOnCurrent = true;
extern bool   alertsMessage   = true;
extern bool   alertsSound     = true;
extern bool   alertsEmail     = false;
extern bool   alertsNotify    = false;
extern string soundFile       = "alert2.wav";

extern bool   ShowT3MaLines   = true;
extern bool   ShowArrows      = TRUE;
extern int    arrowSize       = 1;
extern bool   ArrowsOnFirstBar= true;

//
//
//
//
//

double CrossUp[];
double CrossDn[];
double fastMa[];
double slowMa[];
double trend[];

//
//
//
//
//

string indicatorFileName;
int    timeFrame;
bool   returnBars;
bool   calculateValue;


//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int init()
{

   IndicatorBuffers(5);   
   SetIndexBuffer(0,CrossUp);  
   SetIndexBuffer(1,CrossDn); 
   SetIndexBuffer(2,fastMa);
   SetIndexBuffer(3,slowMa);
   SetIndexBuffer(4,trend);
   
   if (ShowArrows)
   {
     SetIndexStyle(0,DRAW_ARROW,0,arrowSize); SetIndexArrow(0,233);
     SetIndexStyle(1,DRAW_ARROW,0,arrowSize); SetIndexArrow(1,234);   }
   else
   {
     SetIndexStyle(0,DRAW_NONE);
     SetIndexStyle(1,DRAW_NONE);
   }       
   
   
   if (ShowT3MaLines)
   {
     SetIndexStyle(2,DRAW_LINE); 
     SetIndexStyle(3,DRAW_LINE); 
   }
   else
   {
     SetIndexStyle(2,DRAW_NONE);
     SetIndexStyle(3,DRAW_NONE);
   }       
   
         //
         //
         //
         //
         //
         
         T3FastPeriod      = MathMax(T3FastPeriod,1);
         T3SlowPeriod      = MathMax(T3SlowPeriod,1);
         indicatorFileName = WindowExpertName();
         calculateValue    = (TimeFrame=="calculateValue"); if (calculateValue) return(0);
         returnBars        = (TimeFrame=="returnBars");     if (returnBars)     return(0);
         timeFrame         = stringToTimeFrame(TimeFrame);
         
         //
         //
         //
         //
         //
         
   IndicatorShortName(timeFrameToString(timeFrame)+  " T3 Cross ");
return(0);
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int start()
{
   int counted_bars=IndicatorCounted();
   int i,limit;

   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
         limit = MathMin(Bars-counted_bars,Bars-1);
         if (returnBars) { CrossUp[0] = limit+1; return(0); }

   //
   //
   //
   //
   //
   
   if (calculateValue || timeFrame==Period())
   {   
     for(i=limit; i>=0; i--)
     {
        fastMa[i] = iT3(iMA(NULL,0,1,0,MODE_SMA,T3FastPrice,i),T3FastPeriod,T3FastHot,T3FastOriginal,i,0);
        slowMa[i] = iT3(iMA(NULL,0,1,0,MODE_SMA,T3SlowPrice,i),T3SlowPeriod,T3SlowHot,T3SlowOriginal,i,1);
        trend[i]  = trend[i+1];
           if (fastMa[i]>slowMa[i]) trend[i] = 1;
           if (fastMa[i]<slowMa[i]) trend[i] =-1;

           //
           //
           //
           //
           //
      
          CrossUp[i] = EMPTY_VALUE;
          CrossDn[i] = EMPTY_VALUE;
          if (trend[i]!= trend[i+1])
          if (trend[i] == 1)
                CrossUp[i] = Low[i]  - iATR(NULL,0,20,i)/2;
          else  CrossDn[i] = High[i] + iATR(NULL,0,20,i)/2;
         }
         
      manageAlerts();
      return(0);
      }
      
      //
      //
      //
      //
      //
      
      int shift = -1; if (ArrowsOnFirstBar) shift=1;
      limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period()));
      for (i=limit;i>=0; i--)
      {
         int y = iBarShift(NULL,timeFrame,Time[i]);
         int x = iBarShift(NULL,timeFrame,Time[i+shift]);
            fastMa[i]  = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",T3FastPeriod,T3FastPrice,T3FastHot,T3FastOriginal,T3SlowPeriod,T3SlowPrice,T3SlowHot,T3SlowOriginal,2,y);
            slowMa[i]  = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",T3FastPeriod,T3FastPrice,T3FastHot,T3FastOriginal,T3SlowPeriod,T3SlowPrice,T3SlowHot,T3SlowOriginal,3,y);
            trend[i]   = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",T3FastPeriod,T3FastPrice,T3FastHot,T3FastOriginal,T3SlowPeriod,T3SlowPrice,T3SlowHot,T3SlowOriginal,4,y);
         if (x!=y)
         {
            CrossUp[i] = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",T3FastPeriod,T3FastPrice,T3FastHot,T3FastOriginal,T3SlowPeriod,T3SlowPrice,T3SlowHot,T3SlowOriginal,0,y);
            CrossDn[i] = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",T3FastPeriod,T3FastPrice,T3FastHot,T3FastOriginal,T3SlowPeriod,T3SlowPrice,T3SlowHot,T3SlowOriginal,1,y);
         }
         else
         {
            CrossUp[i] = EMPTY_VALUE;
            CrossDn[i] = EMPTY_VALUE;
         }  
      }
  manageAlerts();
return(0);
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

double workT3[][12];
double workT3Coeffs[][6];
#define _period 0
#define _c1     1
#define _c2     2
#define _c3     3
#define _c4     4
#define _alpha  5

//
//
//
//
//

double iT3(double price, double period, double hot, bool original, int i, int instanceNo=0)
{
   if (ArrayRange(workT3,0) != Bars)                ArrayResize(workT3,Bars);
   if (ArrayRange(workT3Coeffs,0) < (instanceNo+1)) ArrayResize(workT3Coeffs,instanceNo+1);

   if (workT3Coeffs[instanceNo][_period] != period)
   {
     workT3Coeffs[instanceNo][_period] = period;
        double a = hot;
            workT3Coeffs[instanceNo][_c1] = -a*a*a;
            workT3Coeffs[instanceNo][_c2] = 3*a*a+3*a*a*a;
            workT3Coeffs[instanceNo][_c3] = -6*a*a-3*a-3*a*a*a;
            workT3Coeffs[instanceNo][_c4] = 1+3*a+a*a*a+3*a*a;
            if (original)
                 workT3Coeffs[instanceNo][_alpha] = 2.0/(1.0 + period);
            else workT3Coeffs[instanceNo][_alpha] = 2.0/(2.0 + (period-1.0)/2.0);
   }
   
   //
   //
   //
   //
   //
   
   int buffer = instanceNo*6;
   int r = Bars-i-1;
   if (r == 0)
      {
         workT3[r][0+buffer] = price;
         workT3[r][1+buffer] = price;
         workT3[r][2+buffer] = price;
         workT3[r][3+buffer] = price;
         workT3[r][4+buffer] = price;
         workT3[r][5+buffer] = price;
      }
   else
      {
         workT3[r][0+buffer] = workT3[r-1][0+buffer]+workT3Coeffs[instanceNo][_alpha]*(price              -workT3[r-1][0+buffer]);
         workT3[r][1+buffer] = workT3[r-1][1+buffer]+workT3Coeffs[instanceNo][_alpha]*(workT3[r][0+buffer]-workT3[r-1][1+buffer]);
         workT3[r][2+buffer] = workT3[r-1][2+buffer]+workT3Coeffs[instanceNo][_alpha]*(workT3[r][1+buffer]-workT3[r-1][2+buffer]);
         workT3[r][3+buffer] = workT3[r-1][3+buffer]+workT3Coeffs[instanceNo][_alpha]*(workT3[r][2+buffer]-workT3[r-1][3+buffer]);
         workT3[r][4+buffer] = workT3[r-1][4+buffer]+workT3Coeffs[instanceNo][_alpha]*(workT3[r][3+buffer]-workT3[r-1][4+buffer]);
         workT3[r][5+buffer] = workT3[r-1][5+buffer]+workT3Coeffs[instanceNo][_alpha]*(workT3[r][4+buffer]-workT3[r-1][5+buffer]);
      }

   //
   //
   //
   //
   //
   
   return(workT3Coeffs[instanceNo][_c1]*workT3[r][5+buffer] + 
          workT3Coeffs[instanceNo][_c2]*workT3[r][4+buffer] + 
          workT3Coeffs[instanceNo][_c3]*workT3[r][3+buffer] + 
          workT3Coeffs[instanceNo][_c4]*workT3[r][2+buffer]);
}

//+-------------------------------------------------------------------
//|                                                                  
//+-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = stringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}

//
//
//
//
//

string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string stringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int tchar = StringGetChar(s, length);
         if((tchar > 96 && tchar < 123) || (tchar > 223 && tchar < 256))
                     s = StringSetChar(s, length, tchar - 32);
         else if(tchar > -33 && tchar < 0)
                     s = StringSetChar(s, length, tchar + 224);
   }
   return(s);
}

//
//
//
//
//

void manageAlerts()
{
   if (!calculateValue && alertsOn)
   {
      if (alertsOnCurrent)
           int whichBar = 0;
      else     whichBar = 1; whichBar = iBarShift(NULL,0,iTime(NULL,timeFrame,whichBar));
      if (trend[whichBar] != trend[whichBar+1])
      {
         if (trend[whichBar] == 1) doAlert(whichBar,"buy");
         if (trend[whichBar] ==-1) doAlert(whichBar,"sell");
      }         
   }
}   

//
//
//
//
//

void doAlert(int forBar, string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
   if (previousAlert != doWhat || previousTime != Time[forBar]) {
       previousAlert  = doWhat;
       previousTime   = Time[forBar];

       //
       //
       //
       //
       //

       message =  StringConcatenate(Symbol()," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," - ",timeFrameToString(timeFrame)+" T3 Cross ",doWhat);
          if (alertsMessage) Alert(message);
          if (alertsNotify)  SendNotification(message);
          if (alertsEmail)   SendMail(StringConcatenate(Symbol()," T3 Cross "),message);
          if (alertsSound)   PlaySound(soundFile);
   }
}

