//+------------------------------------------------------------------+
//|                                                     TmaSlope.mq4 |
//+------------------------------------------------------------------+
// "Lite" = only current timeframe
// mod002 - add alerts
// mod003 - delete repaints at arrows; fix error at popup-alerts

#property copyright "Copyright  2012, zznbrm"

//Edited by shahrooz "sh.sadeghi.me@gmail.com"                           
//---- indicator settings
#property indicator_separate_window
#property  indicator_buffers 7


#property indicator_color1 CLR_NONE
#property indicator_color2 CLR_NONE
#property indicator_color3 CLR_NONE
#property indicator_color4 CLR_NONE
#property indicator_color5 CLR_NONE
#property indicator_color6 CLR_NONE
#property indicator_color7 CLR_NONE


#property indicator_width1 2
#property indicator_width2 2
#property indicator_width3 2
#property indicator_width4 2
#property indicator_width5 2  
#property indicator_width6 2   
#property indicator_width7 0   

//---- constant values ------------------------------------------------------------------------+
#define IndicatorName                   "TmaSlope"
#define Arrow_Signature                 "TmaSlope_arrow"
#define Arrow_Code_Buy                  251
#define Arrow_Code_Sell                 251

//---- input parameters
extern int    eintPeriod = 5;
extern double edblHigh1  = 1.75;
extern double edblLow1   = -1.75;
extern int    atrPeriod  = 10;

extern bool   UseTrendAlert                   = false;  // true = use trend-alerts; false = does not use trend-alerts
extern bool   Trend_Ranging                   = false; // false = do nothing when TMA is Ranging
extern bool   Trend_AtClosedBars              = false; // true = generate alerts by closed bars; false = generate alerts by current bar 
extern bool   Trend_ArrowAlert                = true;  // true = draw arrow on chart at alert
extern int    Trend_Arrow_Width               = 4;     // width of arrow
extern color  Trend_Arrow_Color_Buy           = MediumBlue; // color of Buy-arrow
extern color  Trend_Arrow_Color_Sell          = Red;   // color of Sell-arrow
extern double Trend_Arrow_OffsetPips          = 0;     // offset (in pips) arrow from peak of bar
extern bool   Trend_PopUpAlert                = false;  // true = generate PopUp-window at alert-event
extern bool   Trend_EmailAlert                = false; // true = send e-mail at alert-event
extern bool   Trend_SoundAlert                = false; // true = play sound-file at alert-event
extern string Trend_SoundAlert_FileName_Buy   = "alert.wav"; // name of sound-file (for option SoundAlert)
extern string Trend_SoundAlert_FileName_Sell  = "alert.wav"; // name of sound-file (for option SoundAlert)
extern bool       ShowLabels          = true;
extern color      Font_Color          = White;
extern int        H_Pos               = 1;
extern int        V_Pos               = 25;
extern int        Corner              = 3;
extern int        Font_Size           = 14;
extern int        Font_Size_text      = 11;
extern int        H_Pos_text          = 1;
extern int        V_Pos_text          = 1;
extern int        Text_Corner         = 3;

//---- indicator buffers
double gadblUp1[];
double gadblUp2[];

double gadblDn1[];
double gadblDn2[];

double gadblMid1[];
double gadblMid2[];

double gadblSlope[]; 
color Font_Color_M15;

double TICK;
bool AdditionalDigit;
bool have_init;
int ind_win;
static double   arrow_offset_pips;
static int      Arrow_Code[2];
static color    Arrow_Color[2];
static datetime bartime_alert_buy;
static datetime bartime_alert_sell;
static int      trend_alert_shift;
static int      arrow_last_alert_trend = 0;
static int      last_alert_trend = 0;
static string   str_timeframe;
static string   symbol;

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
{              
   //IndicatorBuffers( 8 );    
   IndicatorBuffers( 7 );
   IndicatorDigits( 5 );
   IndicatorShortName( "TmaSlope_1.5" );
   AdditionalDigit = MarketInfo(Symbol(), MODE_MARGINCALCMODE) == 0 && MarketInfo(Symbol(), MODE_PROFITCALCMODE) == 0 && Digits % 2 == 1;
   TICK = getTick();
      
   SetIndexBuffer( 0, gadblUp1 );    SetIndexLabel( 0, NULL );       SetIndexStyle( 0, DRAW_HISTOGRAM );
   SetIndexBuffer( 1, gadblUp2 );    SetIndexLabel( 1, NULL );       SetIndexStyle( 1, DRAW_HISTOGRAM );
   SetIndexBuffer( 2, gadblDn1 );    SetIndexLabel( 2, NULL );       SetIndexStyle( 2, DRAW_HISTOGRAM );
   SetIndexBuffer( 3, gadblDn2 );    SetIndexLabel( 3, NULL );       SetIndexStyle( 3, DRAW_HISTOGRAM );
   SetIndexBuffer( 4, gadblMid1 );    SetIndexLabel( 4, NULL );       SetIndexStyle( 4, DRAW_HISTOGRAM );
   SetIndexBuffer( 5, gadblMid2 );    SetIndexLabel( 5, NULL );       SetIndexStyle( 5, DRAW_HISTOGRAM );

   SetIndexBuffer( 6, gadblSlope );  SetIndexLabel( 6, "TMA Slope" );    SetIndexStyle( 6, DRAW_NONE );

      
   SetIndexEmptyValue( 0, 0.0 );
   SetIndexEmptyValue( 1, 0.0 );
   SetIndexEmptyValue( 2, 0.0 );

   have_init = true;
   //---- initialization for constants --------------------------------------------------------+
   if(UseTrendAlert)
     {
      if(Digits < 3) double point = 0.01;
      else point = 0.0001;
      if(Trend_AtClosedBars) trend_alert_shift = 1;
      else trend_alert_shift = 0;
      if(Trend_ArrowAlert)
        {
         Arrow_Code [OP_BUY]  = Arrow_Code_Buy;
         Arrow_Code [OP_SELL] = Arrow_Code_Sell;
         Arrow_Color[OP_BUY]  = Trend_Arrow_Color_Buy;
         Arrow_Color[OP_SELL] = Trend_Arrow_Color_Sell;
         arrow_offset_pips = NormalizeDouble(Trend_Arrow_OffsetPips * point,Digits);
        }
      if(Trend_SoundAlert) if(StringLen(Trend_SoundAlert_FileName_Buy) <= 0 && StringLen(Trend_SoundAlert_FileName_Sell) <= 0) Trend_SoundAlert = false;
      UseTrendAlert = Trend_ArrowAlert || Trend_SoundAlert || Trend_EmailAlert || Trend_PopUpAlert;
      str_timeframe = TimeFrameToStr(Period());
      symbol = Symbol();
     }
   return( 0 );
}

//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit()
  {
   if(Trend_ArrowAlert) DeleteObjects(Arrow_Signature);
   if(ShowLabels)
     {
      if(ObjectFind("label")    >= 0) ObjectDelete("label");
      if(ObjectFind("InfoBar1") >= 0) ObjectDelete("InfoBar1");
     }
   return(0);
  }

//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start()
  {
   if(have_init)
     {
      ind_win = WindowFind("TmaSlope_1.5");
      have_init = false;
     }
   int counted_bars = IndicatorCounted();
   if(counted_bars < 0) return(-1);
   if(counted_bars > 0) counted_bars--;
   int limit = Bars - counted_bars;
   int intLimit = MathMin(Bars - 1,Bars - counted_bars + eintPeriod);
   for(int i=intLimit; i>=0; i--)
     {   
      double atr= iATR(NULL,0,atrPeriod,i+10)*0.1;
      if(atr == 0) continue;
      double dblTma  = calcTma(i);
      double dblPrev = calcTma(i+1);
      gadblSlope[i] = (dblTma - dblPrev) / atr;
      gadblUp1  [i] = 0.0;   
      gadblDn1  [i] = 0.0;     
      gadblMid1 [i] = 0.0;   
      gadblUp2  [i] = 0.0;   
      gadblDn2  [i] = 0.0;     
      gadblMid2 [i] = 0.0;   
      if(gadblSlope[i] > edblHigh1)
        {
         if(gadblSlope[i] < gadblSlope[i+1]) gadblUp1[i] = gadblSlope[i];
         else gadblUp2[i] = gadblSlope[i];
        }
      else
        {
         if(gadblSlope[i] < edblLow1)
           {
            if(gadblSlope[i] < gadblSlope[i+1]) gadblDn2[i] = gadblSlope[i];
            else gadblDn1[i] = gadblSlope[i];
           }
         else  
           {
            if(gadblSlope[i] < gadblSlope[i+1]) gadblMid2[i] = gadblSlope[i];
            else gadblMid1[i] = gadblSlope[i];
           } 
        }
     }
   //---- trend-alerts ------------------------------------------------------------------------+
   if(UseTrendAlert)
     {
      if(Trend_ArrowAlert)
        {
         for(i=limit; i>=trend_alert_shift; i--)
           {
            int signal = -1;
            if(gadblSlope[i] >= edblHigh1)
              {
               if(arrow_last_alert_trend <= 0)
                 {
                  arrow_last_alert_trend = 1;
                  signal = OP_BUY;
                 }
              }
            else
              {
               if(gadblSlope[i] <= edblLow1)
                 {
                  if(arrow_last_alert_trend >= 0)
                    {
                     arrow_last_alert_trend = -1;
                     signal = OP_SELL;
                    }
                 }
               else if(Trend_Ranging) arrow_last_alert_trend = 0;
              }
            if(signal < 0) continue;
            if(signal == OP_BUY) double obj_price1 = NormalizeDouble(Low[i] - arrow_offset_pips,Digits);
            else obj_price1 = NormalizeDouble(High[i] + arrow_offset_pips,Digits);
            CreateObjectArrow(StringConcatenate(Arrow_Signature," ",OrderTypeToStr(signal)," ",TimeToStr(Time[i])),Time[i],obj_price1,Arrow_Code[signal],Arrow_Color[signal],Trend_Arrow_Width);
           }
        }
      signal = 0;
      if(gadblSlope[trend_alert_shift] >= edblHigh1)
        {
         if(last_alert_trend <= 0)
           {
            last_alert_trend = 1;
            signal++;
           }
        }
      else
        {
         if(gadblSlope[trend_alert_shift] <= edblLow1)
           {
            if(last_alert_trend >= 0)
              {
               last_alert_trend = -1;
               signal--;
              }
           }
         else if(Trend_Ranging) last_alert_trend = 0;
        }
      if(signal != 0)
        {
         if(signal > 0)
           {
            if(bartime_alert_buy != Time[0])
              {
               bartime_alert_buy = Time[0];
               if(Trend_SoundAlert) if(StringLen(Trend_SoundAlert_FileName_Buy) > 0) PlaySound(Trend_SoundAlert_FileName_Buy);
               if(Trend_EmailAlert) SendMail(StringConcatenate("Alert from indicator \"",IndicatorName,"\": ",symbol," ",str_timeframe," TREND ALERT BUY"),StringConcatenate(IndicatorName,": ",symbol," ",str_timeframe," TREND ALERT BUY"));
               if(Trend_PopUpAlert) Alert(StringConcatenate(IndicatorName,": ",symbol," ",str_timeframe," TREND ALERT BUY"));
              }
           }
         else
           {
            if(bartime_alert_sell != Time[0])
              {
               bartime_alert_sell = Time[0];
               if(Trend_SoundAlert) if(StringLen(Trend_SoundAlert_FileName_Sell) > 0) PlaySound(Trend_SoundAlert_FileName_Sell);
               if(Trend_EmailAlert) SendMail(StringConcatenate("Alert from indicator \"",IndicatorName,"\": ",symbol," ",str_timeframe," TREND ALERT SELL"),StringConcatenate(IndicatorName,": ",symbol," ",str_timeframe," TREND ALERT SELL"));
               if(Trend_PopUpAlert) Alert(StringConcatenate(IndicatorName,": ",symbol," ",str_timeframe," TREND ALERT SELL"));
              }
           }
        }
     }
   //---- 
   if(ShowLabels)
     {
      string tt = DoubleToStr(gadblSlope[0],2);
      ObjectCreate("label",OBJ_LABEL,ind_win,0,0);
      ObjectSet("label",OBJPROP_XDISTANCE,H_Pos);
      ObjectSet("label",OBJPROP_YDISTANCE,V_Pos);
      ObjectSet("label",OBJPROP_CORNER,Corner);
      ObjectSetText("label"," "+tt+" ",Font_Size,"Arial",Font_Color);
      string sObjName="InfoBar1";
      ObjectCreate(sObjName,OBJ_LABEL,ind_win,0,0);
      ObjectSet(sObjName,OBJPROP_CORNER,Text_Corner);
      ObjectSet(sObjName,OBJPROP_XDISTANCE,H_Pos_text);//left to right
      ObjectSet(sObjName,OBJPROP_YDISTANCE,V_Pos_text);//top to bottom
      if(gadblSlope[0] >= edblHigh1) ObjectSetText(sObjName,"Buy Only",Font_Size_text,"Verdana",Lime);
      else if(gadblSlope[0] <= edblLow1) ObjectSetText(sObjName,"Sell Only",Font_Size_text,"Verdana",Red);
           else ObjectSetText(sObjName,"Ranging",Font_Size_text,"Verdana",Yellow);
     }
   return(0);
  }

//+------------------------------------------------------------------+
//| getTick()                                                        |
//+------------------------------------------------------------------+
double getTick() {
    double tick = MarketInfo(Symbol(), MODE_TICKSIZE);
    if (AdditionalDigit) {
        tick *= 10;
    }    
    return (tick);
}

//+------------------------------------------------------------------+
//| calcTma()                                                        |
//+------------------------------------------------------------------+
double calcTma( int inx )
{
   double dblSum  = (eintPeriod+1)*Close[inx];
   double dblSumw = (eintPeriod+1);
   int jnx, knx;
         
   for ( jnx = 1, knx = eintPeriod; jnx <= eintPeriod; jnx++, knx-- )
   {
      dblSum  += ( knx * Close[inx+jnx] );
      dblSumw += knx;

      if ( jnx <= inx )
      {
         dblSum  += ( knx * Close[inx-jnx] );
         dblSumw += knx;
      }
   }
   
   return( dblSum / dblSumw );
}
 
//---------------------------------------------------------------------------------------------+
//---- DeleteObjects --------------------------------------------------------------------------+
//-------------------------------- programming by Serega Lykov, http://mtexperts.narod.ru/ ----+
//---------------------------------------------------------------------------------------------+
void DeleteObjects(string obj_name_signature, int obj_type=-1)
  {
   int obj_total = ObjectsTotal() - 1;
   for(int i=obj_total; i>=0; i--)
     {
      string obj_name = ObjectName(i);
      if(obj_type >= 0 && ObjectType(obj_name) != obj_type) continue;
      if(StringFind(obj_name,obj_name_signature,0) != 0) continue;
      ObjectDelete(obj_name);
     }
  }

//---------------------------------------------------------------------------------------------+
//---- CreateObjectArrow ----------------------------------------------------------------------+
//-------------------------------- programming by Serega Lykov, http://mtexperts.narod.ru/ ----+
//---------------------------------------------------------------------------------------------+
void CreateObjectArrow(string obj_name, datetime obj_time1, double obj_price1, int obg_arrowcode, color obj_color, int obj_width, int obj_window = 0)
  {
   if(ObjectFind(obj_name) < 0) ObjectCreate(obj_name,OBJ_ARROW,obj_window,obj_time1,obj_price1);
   else
     {
      if(ObjectGet(obj_name,OBJPROP_TIME1) != obj_time1 || NormalizeDouble(ObjectGet(obj_name,OBJPROP_PRICE1),Digits) != obj_price1)
        {
         ObjectMove(obj_name,0,obj_time1,obj_price1);
        }
     }
   ObjectSet(obj_name,OBJPROP_ARROWCODE,obg_arrowcode);
   ObjectSet(obj_name,OBJPROP_WIDTH,obj_width);
   ObjectSet(obj_name,OBJPROP_COLOR,obj_color);
  }

//---------------------------------------------------------------------------------------------+
//---- OrderTypeToStr -------------------------------------------------------------------------+
//-------------------------------- programming by Serega Lykov, http://mtexperts.narod.ru/ ----+
//---------------------------------------------------------------------------------------------+
string OrderTypeToStr(int order_type)
  {
   switch(order_type)
     {
      case OP_BUY: string result = "Buy";       break;
      case OP_SELL:       result = "Sell";      break;
      case OP_BUYLIMIT:   result = "BuyLimit";  break;
      case OP_SELLLIMIT:  result = "SellLimit"; break;
      case OP_BUYSTOP:    result = "BuyStop";   break;
      case OP_SELLSTOP:   result = "SellStop";  break;
      default:            result = "Unknown";   break;
     }
   return(result);
  }

//---------------------------------------------------------------------------------------------+
//---- TimeFrameToStr -------------------------------------------------------------------------+
//-------------------------------- programming by Serega Lykov, http://mtexperts.narod.ru/ ----+
//---------------------------------------------------------------------------------------------+
string TimeFrameToStr(int period)
  {
   switch(period)
     {
      case     0: string str_timeframe = TimeFrameToStr(Period()); break;
      case     1:        str_timeframe = "M1";          break;
      case     5:        str_timeframe = "M5";          break;
      case    15:        str_timeframe = "M15";         break;
      case    30:        str_timeframe = "M30";         break;
      case    60:        str_timeframe = "H1";          break;
      case   240:        str_timeframe = "H4";          break;
      case  1440:        str_timeframe = "D1";          break;
      case 10080:        str_timeframe = "W1";          break;
      case 43200:        str_timeframe = "MN";          break;
      default:           str_timeframe = StringConcatenate("M",DoubleToStr(period,0));
     }
   return(str_timeframe);
  }

 
 
 
 
 
 
 
 
 
 
 



   


