/*------------------------------------------------------------------------------------
   Name: TMS-Osc Cross.mq4
   Copyright 2010, Xaphod, http://www.xaphod.com
   
   Description: Oscillator line cross indicator for the Trading Made Simple System
                Similar to the TDI RSI Signal Line and Trade Signal Line.
	          
   Change log: 
       2015-01-09. Xaphod, v1.601
          - Draw arrows on open bar too.
          - Parameter for alert trigger bar
       2014-05-31. Xaphod, v1.600 
          - Update for build 600
          - Added push notification for alerts
       2011-05-18. Xaphod, v1.04
          - Fix for repeated alerts
       2011-05-17. Xaphod, v1.03
          - Timestamp in email alert corrected
       2011-05-17. Xaphod, v1.02 
          - Alerts for bar 1 only. Prevents multiple alerts when MT4 reloads bars
       2011-05-16. Xaphod, v1.01 
          - No alerts after initialization
       2011-05-16. Xaphod, v1.00 
          - First Release 
-------------------------------------------------------------------------------------*/
// Indicator properties
#property copyright "2010, xaphod"
#property link      "http://www.xaphod.com"

#property strict
#property version    "1.601"
#property description "Oscillator line cross indicator for the Trading Made Simple System."
#property description "Similar to the TDI RSI Signal Line and Trade Signal Line."
#property description " "
#property description "Troubleshooting: Check the 'Journal' and 'Expert' tabs in the terminal window for errors."

#property indicator_chart_window
#property indicator_buffers 5
#property indicator_color1 CLR_NONE
#property indicator_width1 1
#property indicator_color2 CLR_NONE
#property indicator_width2 2
#property indicator_color3 CLR_NONE
#property indicator_width3 2
#property indicator_color4 LimeGreen
#property indicator_width4 1
#property indicator_color5 Red
#property indicator_width5 1

//#include <xDebug.mqh>

// Constant definitions
#define INDICATOR_NAME "TMS-Osc Cross"

// Signals
#define SIG_NO 0
#define SIG_UP 1
#define SIG_DN 2
// Arrows
#define ARROW_THIN_UP 225
#define ARROW_THIN_DN 226
#define ARROW_THICK_UP 233
#define ARROW_THICK_DN 234
#define ARROW_HOLLOW_UP 241
#define ARROW_HOLLOW_DN 242
#define ARROW_ROUND_UP 221
#define ARROW_ROUND_DN 222
#define ARROW_DIAG_THIN_UP 228
#define ARROW_DIAG_THIN_DN 230
#define ARROW_DIAG_THICK_UP 236
#define ARROW_DIAG_THICK_DN 238
#define ARROW_DIAG_HOLLOW_UP 246
#define ARROW_DIAG_HOLLOW_DN 248
#define ARROW_FRACTAL_UP 217
#define ARROW_FRACTAL_DN 218
#define ARROW_RIBBON_UP 200
#define ARROW_RIBBON_DN 202
#define ARROW_THUMB_UP 67
#define ARROW_THUMB_DN 68
#define ARROW_FINGER_UP 67
#define ARROW_FINGER_DN 68

enum ENUM_ARROW_TYPE {
   Thin_Arrow=1,
   Thick_Arrow=2, 
   Hollow_Arrow=3, 
   Round_Arrow=4, 
   Diagonal_Thin_Arrow=5, 
   Diagonal_Thick_Arrow=6, 
   Diagonal_Hollow_Arrow=7, 
   Fractal_Char=8, 
   Ribbon_Char=9, 
   Thumb_Char=10, 
   Finger_Char=11
};

// Indicator parameters
extern int                RSI_Period=13;               /* RSI Period */     
extern ENUM_APPLIED_PRICE RSI_Price=PRICE_CLOSE;       /* RSI Price Type */   
extern int                RSISignal_Period=2;          /* RSI Signal Period */   
extern ENUM_MA_METHOD     RSISignal_Method=MODE_SMA;   /* RSI Signal MA Method */
extern int                TradeSignal_Period=7;        /* Trade Signal Period */
extern ENUM_MA_METHOD     TradeSignal_Method=MODE_SMA; /* Trade Signal MA Method */
extern ENUM_ARROW_TYPE    Arrow_Type=Thick_Arrow;      /* Arrow Character */
extern int                Arrow_Size=1;                /* Arrow Size (0-5) */ 
extern int                Arrow_Spacing=5;             /* Arrow Offset from Candle */ 
extern int                Alert_Bar=1;                 /* Alert: Trigger Bar */ 
extern bool               Alert_Popup=false;           /* Alert: Enable Popup with Sound */ 
extern bool               Alert_Notification=false;    /* Alert: Enable Push Notification */ 
extern bool               Alert_Email=false;           /* Alert: Send email */
extern string             Alert_Subject="Default";     /* Email Subject. */ //If "Default" the subject is set by the indicator


// Global module varables
double gadRSI[];
double gadRSISig[];
double gadTradeSig[];
double gadArrowUp[];
double gadArrowDn[];
bool gbInit;

//-----------------------------------------------------------------------------
// function: init()
// Description: Custom indicator initialization function.
//-----------------------------------------------------------------------------
int init() {
  int iArrowUp;
  int iArrowDn;  
  
  SetIndexStyle(0, DRAW_LINE);
  SetIndexBuffer(0, gadRSI);
  SetIndexLabel(0,NULL);
  SetIndexStyle(1, DRAW_LINE);
  SetIndexBuffer(1, gadRSISig);
  SetIndexLabel(1,"RSI Signal");
  SetIndexStyle(2, DRAW_LINE);
  SetIndexBuffer(2, gadTradeSig);
  SetIndexLabel(2,"Trade Signal");
  SetArrowType(iArrowUp, iArrowDn );
  SetIndexStyle(3,DRAW_ARROW,EMPTY,Arrow_Size);
  SetIndexBuffer(3,gadArrowUp);
  SetIndexLabel(3,"Trend Up");
  SetIndexArrow(3, iArrowUp);
  SetIndexStyle(4,DRAW_ARROW,EMPTY,Arrow_Size);
  SetIndexBuffer(4,gadArrowDn);
  SetIndexLabel(4,"Trend Down");
  SetIndexArrow(4, iArrowDn);

  gbInit=True;
  
  IndicatorDigits(1);
  IndicatorShortName(INDICATOR_NAME);
  return(0);
}


//-----------------------------------------------------------------------------
// function: deinit()
// Description: Custom indicator deinitialization function.
//-----------------------------------------------------------------------------
int deinit() {
   return (0);
}


///-----------------------------------------------------------------------------
// function: start()
// Description: Custom indicator iteration function.
//-----------------------------------------------------------------------------
int start() {
  int iNewBars;
  int iCountedBars; 
  int i;
    
  // Get unprocessed ticks
  iCountedBars=IndicatorCounted();
  if(iCountedBars < 0) return (-1); 
  if(iCountedBars>0) iCountedBars--;
  iNewBars=MathMin(Bars-iCountedBars, Bars-RSI_Period-1);
  
  
  // Calc TMS Signals
  for(i=iNewBars; i>=0; i--) {
    gadRSI[i] = iRSI(NULL,0,RSI_Period,RSI_Price,i); 
  }  
  for(i=iNewBars; i>=0; i--) {  
    gadRSISig[i]=iMAOnArray(gadRSI,0,RSISignal_Period,0,RSISignal_Method,i);
    gadTradeSig[i]=iMAOnArray(gadRSI,0,TradeSignal_Period,0,TradeSignal_Method,i);    
  }
  
  // Check for cross signal
  for(i=iNewBars; i>=0; i--) {
  
    // Clear indicator buffer 
    gadArrowUp[i]=EMPTY_VALUE;
    gadArrowDn[i]=EMPTY_VALUE;
  
    // Cross Up 
    if ((gadRSISig[i] > gadTradeSig[i]) && (gadRSISig[i+1] < gadTradeSig[i+1])) 
      gadArrowUp[i]=Low[i]-GetArrowOffset(Time[i], High[i]);
    // Cross Down
    else if ((gadRSISig[i] < gadTradeSig[i]) && (gadRSISig[i+1] > gadTradeSig[i+1]))
      gadArrowDn[i]=High[i]+GetArrowOffset(Time[i], High[i]);
  }
  
  // Check for alert alert
  CrossAlert(MathMax(Alert_Bar,0));
  
  // Clear Init Flag
  if (gbInit)
    gbInit=False;
    
  return(0);
}
//+------------------------------------------------------------------+





//-----------------------------------------------------------------------------
// function: CrossDrawArrow()
// Description: Draw arrows for signal crosses
//-----------------------------------------------------------------------------
int CrossDrawArrow(int iBar, double dSpacing) { 
  // Clear indicator buffer 
  gadArrowUp[iBar]=EMPTY_VALUE;
  gadArrowDn[iBar]=EMPTY_VALUE;
  
  // Up signal 
  if ((gadRSISig[iBar] > gadTradeSig[iBar]) && (gadRSISig[iBar+1] < gadTradeSig[iBar+1])) 
    gadArrowUp[iBar]=Low[iBar]-dSpacing;
  // Down signal
  else if ((gadRSISig[iBar] < gadTradeSig[iBar]) && (gadRSISig[iBar+1] > gadTradeSig[iBar+1]))
    gadArrowDn[iBar]=High[iBar]+dSpacing;  
  return(0);  
}


//-----------------------------------------------------------------------------
// function: CrossAlert()
// Description: Alert on signal crosses
//-----------------------------------------------------------------------------
void CrossAlert(int iBar) {
  static datetime tCurBar=0;
  
  // Do not alert on startup
  if (gbInit) {
    tCurBar=Time[iBar];
    return;
  }
  
  // Alert once per bar only. Ie. one alert on the open bar which can change multiple times.
  if (tCurBar<Time[iBar]) {
    // Up signal     
    if ((gadRSISig[iBar] > gadTradeSig[iBar]) && (gadRSISig[iBar+1] < gadTradeSig[iBar+1])) {
      tCurBar=Time[iBar];
      DoAlert("LONG Signal!");
      
    }
    // Down signal
    else if ((gadRSISig[iBar] < gadTradeSig[iBar]) && (gadRSISig[iBar+1] > gadTradeSig[iBar+1])) {
      tCurBar=Time[iBar];
      DoAlert("SHORT Signal!");
    }
  } 
}


//-----------------------------------------------------------------------------
// function: DoAlert()
// Description: Alert on signal
//-----------------------------------------------------------------------------
void DoAlert(string sAlertMsg) {
  // Do Alerts        
  sAlertMsg=Symbol()+", "+TF2Str(Period())+": "+sAlertMsg;
  if (Alert_Popup) 
    Alert(sAlertMsg, ", ", INDICATOR_NAME);
  if (Alert_Notification) 
    SendNotification(sAlertMsg + ", "+ INDICATOR_NAME);
  if (Alert_Email) {
    if (Alert_Subject=="Default") 
      Alert_Subject=sAlertMsg;
    SendMail( Alert_Subject, "MT4 Alert! "+INDICATOR_NAME+"\n" + TimeToStr(Time[0],TIME_DATE|TIME_SECONDS )+"\n"+sAlertMsg);              
  }
}


//-----------------------------------------------------------------------------
// function: GetArrowOffset()
// Description: Get arrow spacing based on viewable chart area
//-----------------------------------------------------------------------------
double GetArrowOffset(datetime tTime, double dPrice) {
  int iX;
  int iY;
  int iWindow=0;
  datetime tOutTime=0;
  double dOutPrice=0;

  if (ChartTimePriceToXY(0,iWindow,tTime,dPrice,iX,iY)) {
    if (ChartXYToTimePrice(0,iX,iY+Arrow_Spacing,iWindow,tOutTime,dOutPrice)) {
      return(MathAbs(dOutPrice-dPrice));
    }
  }
  return(Point*10);
}
  


//-----------------------------------------------------------------------------
// function: TF2Str()
// Description: Convert time-frame to a string
//-----------------------------------------------------------------------------
string TF2Str(int iPeriod) {
  switch(iPeriod) {
    case PERIOD_M1: return("M1");
    case PERIOD_M5: return("M5");
    case PERIOD_M15: return("M15");
    case PERIOD_M30: return("M30");
    case PERIOD_H1: return("H1");
    case PERIOD_H4: return("H4");
    case PERIOD_D1: return("D1");
    case PERIOD_W1: return("W1");
    case PERIOD_MN1: return("MN1");
    default: return("M"+(string)iPeriod);
  }
}


//-----------------------------------------------------------------------------
// function: SetArrowType()
// Description: Set the arrow character
//-----------------------------------------------------------------------------
void SetArrowType(int& iArrowUp, int& iArrowDn) {
  switch (Arrow_Type) {
    case 1:  // Thin
      iArrowDn=ARROW_THIN_DN;
      iArrowUp=ARROW_THIN_UP;
    break;
    case 2:  // Hollow
      iArrowDn=ARROW_HOLLOW_DN;
      iArrowUp=ARROW_HOLLOW_UP;
    break;
    case 3:  // Round
      iArrowDn=ARROW_ROUND_DN;
      iArrowUp=ARROW_ROUND_UP;
    break;    
    case 4: // Thick
      iArrowDn=ARROW_THICK_DN;
      iArrowUp=ARROW_THICK_UP;
    break;
    case 5: // Diagonal Thin
      iArrowDn=ARROW_DIAG_THIN_DN;
      iArrowUp=ARROW_DIAG_THIN_UP;
    break;
    case 6: // Diagonal Thin
      iArrowDn=ARROW_DIAG_THICK_DN;
      iArrowUp=ARROW_DIAG_THICK_UP;
    break;
    case 7: // Diagonal Hollow
      iArrowDn=ARROW_DIAG_THICK_DN;
      iArrowUp=ARROW_DIAG_THICK_UP;
    break;
    case 8: // Fractal
      iArrowDn=ARROW_FRACTAL_DN;
      iArrowUp=ARROW_FRACTAL_UP;
    break;
    case 9: // Ribbon
      iArrowDn=ARROW_RIBBON_DN;
      iArrowUp=ARROW_RIBBON_UP;
    break;
    case 10: // Thumbs
      iArrowDn=ARROW_THUMB_DN;
      iArrowUp=ARROW_THUMB_UP;
    break;
    case 11: // Finger
      iArrowDn=ARROW_FINGER_DN;
      iArrowUp=ARROW_FINGER_UP;
    break;
    default: // Thick
      iArrowDn=ARROW_THICK_DN;
      iArrowUp=ARROW_THICK_UP;
    break;
  }
}


