//------------------------------------------------------------------
//
//
//
//------------------------------------------------------------------
#property indicator_separate_window
#property indicator_buffers 4
#property indicator_color1  clrDodgerBlue
#property indicator_color2  clrSandyBrown
#property indicator_color3  clrSandyBrown
#property indicator_color4  clrDarkGray
#property indicator_width1  2
#property indicator_width2  2
#property indicator_width3  2
#property indicator_maximum  110
#property indicator_minimum -110
#property indicator_level1   0
#property strict

//
//
//
//
//

enum enTimeFrames
{
   tf_cu  = 0,              // Current time frame
   tf_m1  = PERIOD_M1,      // 1 minute
   tf_m5  = PERIOD_M5,      // 5 minutes
   tf_m15 = PERIOD_M15,     // 15 minutes
   tf_m30 = PERIOD_M30,     // 30 minutes
   tf_h1  = PERIOD_H1,      // 1 hour
   tf_h4  = PERIOD_H4,      // 4 hours
   tf_d1  = PERIOD_D1,      // Daily
   tf_w1  = PERIOD_W1,      // Weekly
   tf_mn1 = PERIOD_MN1      // Monthly
};

extern enTimeFrames       TimeFrame  = tf_cu;   // Time frame to use
extern int                Fast       = 8;       // Fast macd period
extern int                Slow       = 21;      // Slow macd period
extern int                SignalPeriod = 5;     // Signal period
extern ENUM_MA_METHOD     Mode_Smooth  = 2;     // Signal average method
extern ENUM_APPLIED_PRICE PriceToUse   = PRICE_CLOSE; // Price to use
extern int                First_R    = 20;      // First smoothing period
extern int                Second_S   = 5;       // Second smoothing period
extern bool               Interpolate= true;

double TSI_Buffer[];
double TSI_Bufferda[];
double TSI_Bufferdb[];
double SignalBuffer[];
double cross[];
string indicatorFileName;
bool   returnBars;

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

int init()
{
   IndicatorBuffers(5);
   SetIndexBuffer(0,TSI_Buffer);   SetIndexLabel(0, "TSI_MACD");
   SetIndexBuffer(1,TSI_Bufferda); SetIndexLabel(1, "TSI_MACD");
   SetIndexBuffer(2,TSI_Bufferdb); SetIndexLabel(2, "TSI_MACD");
   SetIndexBuffer(3,SignalBuffer); SetIndexLabel(3, "Signal");
   SetIndexBuffer(4,cross);
        indicatorFileName = WindowExpertName();
        returnBars        = TimeFrame == -99;
        TimeFrame         = MathMax(TimeFrame,_Period);   

   IndicatorShortName(timeFrameToString(TimeFrame)+" TSI_MACD(" + (string)Fast + ", " + (string)Slow + " ; " + (string)First_R + ", " + (string)Second_S + ", " + (string)SignalPeriod + ")");
   return(0);
}
int deinit() { return(0); }
int start()
  {
   int i,counted_bars=IndicatorCounted();
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
            int limit=MathMin(Bars-counted_bars,Bars-2);
            if (returnBars) { TSI_Buffer[0] = limit+1; return(0); }
            if (TimeFrame!=_Period)
            {
               limit = (int)MathMax(limit,MathMin(Bars-2,iCustom(NULL,TimeFrame,indicatorFileName,-99,0,0)*TimeFrame/Period())); 
               if (cross[limit]==-1) CleanPoint(limit,TSI_Bufferda,TSI_Bufferdb);
               for(i=limit; i>=0; i--)
               {
                  int y = iBarShift(NULL,TimeFrame,Time[i]);
                     TSI_Buffer[i]   = iCustom(NULL,TimeFrame,indicatorFileName,tf_cu,Fast,Slow,SignalPeriod,Mode_Smooth,PriceToUse,First_R,Second_S,0,y);
                     SignalBuffer[i] = iCustom(NULL,TimeFrame,indicatorFileName,tf_cu,Fast,Slow,SignalPeriod,Mode_Smooth,PriceToUse,First_R,Second_S,3,y);
                     cross[i]        = iCustom(NULL,TimeFrame,indicatorFileName,tf_cu,Fast,Slow,SignalPeriod,Mode_Smooth,PriceToUse,First_R,Second_S,4,y);
                     TSI_Bufferda[i] = EMPTY_VALUE;
                     TSI_Bufferdb[i] = EMPTY_VALUE;

                  //
                  //
                  //
                  //
                  //
      
                     if (!Interpolate || (i>0 && y==iBarShift(NULL,TimeFrame,Time[i-1]))) continue;

                  //
                  //
                  //
                  //
                  //

                     int n,k; datetime time = iTime(NULL,TimeFrame,y);
                        for(n = 1; i+n<Bars && Time[i+n] >= time; n++) continue;	
                        for(k = 1; i+n<Bars && i+k<Bars && k<n; k++) {
                           TSI_Buffer[i+k]   = TSI_Buffer[i]   + (TSI_Buffer[i+n]   - TSI_Buffer[i]  )*k/n;
                           SignalBuffer[i+k] = SignalBuffer[i] + (SignalBuffer[i+n] - SignalBuffer[i])*k/n; }
               }
               for(i=limit; i>=0; i--) if (cross[i]==-1) PlotPoint(i,TSI_Bufferda,TSI_Bufferdb,TSI_Buffer);
               return(0);
            }   

   //
   //
   //
   //
   //
   
   if (cross[limit]==-1) CleanPoint(limit,TSI_Bufferda,TSI_Bufferdb);
   for(i=limit; i>=0; i--)
   {
       double macdMom = iMACD(NULL,0,Fast,Slow,1,PriceToUse,MODE_MAIN,i)-iMACD(NULL,0,Fast,Slow,1,PriceToUse,MODE_MAIN,i+1);
       double div     = iEma(iEma(MathAbs(macdMom),First_R,i,2),Second_S,i,3);
         if ( div!=0)
               TSI_Buffer[i] = 100*iEma(iEma(macdMom,First_R,i,0),Second_S,i,1)/div;
         else  TSI_Buffer[i] = 0;
   }      
   for(i=limit; i>=0; i--) 
   {
      SignalBuffer[i] = iMAOnArray(TSI_Buffer,0,SignalPeriod,0,Mode_Smooth,i);
      TSI_Bufferda[i] = EMPTY_VALUE;
      TSI_Bufferdb[i] = EMPTY_VALUE;
      cross[i] = cross[i+1];
         if (TSI_Buffer[i]>SignalBuffer[i]) cross[i] =  1;
         if (TSI_Buffer[i]<SignalBuffer[i]) cross[i] = -1;
         if (cross[i] ==-1)  if (cross[i]==-1) PlotPoint(i,TSI_Bufferda,TSI_Bufferdb,TSI_Buffer);
   }         
   return(0);
}

//-------------------------------------------------------------------
//                                                                  
//-------------------------------------------------------------------
//
//
//
//
//

double workEma[][4];
double iEma(double price, double period, int r, int instanceNo=0)
{
   if (ArrayRange(workEma,0)!= Bars) ArrayResize(workEma,Bars); r=Bars-r-1;

   workEma[r][instanceNo] = price;
   double alpha = 2.0 / (1.0+period);
      if (r>0)
          workEma[r][instanceNo] = workEma[r-1][instanceNo]+alpha*(price-workEma[r-1][instanceNo]);
   return(workEma[r][instanceNo]);
}

// 
//
//
//
//

string sTfTable[] = {"M1","M5","M10","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,10,15,30,60,240,1440,10080,43200};

string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//-------------------------------------------------------------------
//                                                                  
//-------------------------------------------------------------------
//
//
//
//
//

void CleanPoint(int i,double& first[],double& second[])
{
   if (i>=Bars-3) return;
   if ((second[i]  != EMPTY_VALUE) && (second[i+1] != EMPTY_VALUE))
        second[i+1] = EMPTY_VALUE;
   else
      if ((first[i] != EMPTY_VALUE) && (first[i+1] != EMPTY_VALUE) && (first[i+2] == EMPTY_VALUE))
          first[i+1] = EMPTY_VALUE;
}

void PlotPoint(int i,double& first[],double& second[],double& from[])
{
   if (i>=Bars-2) return;
   if (first[i+1] == EMPTY_VALUE)
      if (first[i+2] == EMPTY_VALUE) 
            { first[i]  = from[i];  first[i+1]  = from[i+1]; second[i] = EMPTY_VALUE; }
      else  { second[i] =  from[i]; second[i+1] = from[i+1]; first[i]  = EMPTY_VALUE; }
   else     { first[i]  = from[i];                           second[i] = EMPTY_VALUE; }
}