//+------------------------------------------------------------------+
//|                                                 zigzagPoints.mq4 |
//|                        Copyright 2016, MetaQuotes Software Corp. |
//|                                             https://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2016, MetaQuotes Software Corp."
#property link      "https://www.mql5.com"
#property version   "1.00"
#property strict
#property indicator_chart_window
#property indicator_buffers 2
#property indicator_plots   2
//--- plot TOP
#property indicator_label1  "TOP"
#property indicator_type1   DRAW_ARROW
#property indicator_color1  clrRed
#property indicator_style1  STYLE_SOLID
#property indicator_width1  1
//--- plot BOTTOM
#property indicator_label2  "BOTTOM"
#property indicator_type2   DRAW_ARROW
#property indicator_color2  clrGreen
#property indicator_style2  STYLE_SOLID
#property indicator_width2  1
//--- indicator buffers
double         TOPBuffer[];
double         BOTTOMBuffer[];
enum TREND_MODE
  {
   REGULAR=0 , // Regular Trend 
   FREE =1 // Free Trend /Channel
  };
input TREND_MODE TM=FREE ; // Trend Lines Drawing Method
input int Step=6 ;
input int Deviation=5;
input int back_step=3 ;
input color up_color=clrGreen ; // UP Trend Line Color
input color dn_color=clrRed ; // Down Trend Line Color 
input bool  Sound_Alert=true ; // Allow Sound Alert
input bool  Push_Alert=true ; // Allow Push Notification
input bool  Email_Alert=true ; // Allow Email Alert

int CUR_BAR=0 ;

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int OnInit()
  {


//--- indicator buffers mapping
   SetIndexBuffer(0,TOPBuffer);
   SetIndexBuffer(1,BOTTOMBuffer);
//--- setting a code from the Wingdings charset as the property of PLOT_ARROW
   PlotIndexSetInteger(0,PLOT_ARROW,159);
   PlotIndexSetInteger(1,PLOT_ARROW,159);
   if(Step<=back_step)
   {
   Print("Step Can't be Less Than or Equal to Back Step");
   return(INIT_FAILED);
   }
//---
   return(INIT_SUCCEEDED);
  }
//--------------------------------
void OnDeinit(const int reason)
  {
   ObjectDelete(ChartID(),"Up_Trend");
   ObjectDelete(ChartID(),"Dn_Trend");
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime &time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[])
  {
//---

     if(CUR_BAR==Bars)return(0);
     
     for(int i=0;i<rates_total-1;i++)
       {
          double Cur_Zigzag=iCustom(Symbol(),Period(),"ZigZag",Step,Deviation,back_step,0,i);
          if(Cur_Zigzag>0)
          {
            for(int x=i+1;x<rates_total-2;x++)
              {
                double Pre_ZigZag=iCustom(Symbol(),Period(),"ZigZag",Step,Deviation,back_step,0,x);
                if(Pre_ZigZag>0 && Cur_Zigzag>Pre_ZigZag)
                {
                 BOTTOMBuffer[x]=Pre_ZigZag ;
                 break;
                }
                else if(Pre_ZigZag>0 && Cur_Zigzag<Pre_ZigZag)
                {
                 TOPBuffer[x]=Pre_ZigZag;
                 break;
                }
              }
          }
       }
    double p1,p2 ;
    datetime d1,d2 ;
    for(int i=0;i<rates_total-3;i++)
      {
        if(BOTTOMBuffer[i]!=EMPTY_VALUE) 
         {
          d2=Time[i] ;
          p2=BOTTOMBuffer[i] ;
          
          for(int x=i+1;x<Bars-4;x++)
            {
              if(BOTTOMBuffer[x]!=EMPTY_VALUE)
              {
                 if(TM==FREE)
                 {
                 d1=Time[x];
                 p1=BOTTOMBuffer[x];
                 break;  
                 }
                 else if(TM==REGULAR && BOTTOMBuffer[x]<p2 )
                 {
                 d1=Time[x];
                 p1=BOTTOMBuffer[x];
                 break;  
                 } 
              }
            }
            break;
         }
      }
       ObjectDelete("Up_Trend");
       Draw_Trend("Up_Trend",p1,d1,p2,d2,up_color);

      
      for(int i=0;i<rates_total-3;i++)
       {
        if(TOPBuffer[i]!=EMPTY_VALUE)
        {
         d2=Time[i];
         p2=TOPBuffer[i];
         for(int x=i+1;x<Bars-4;x++)
            {
              if(TOPBuffer[x]!=EMPTY_VALUE)
              {
                 if(TM==FREE)
                 {
                 d1=Time[x];
                 p1=TOPBuffer[x];
                 break;
                 }
                 else if(TM==REGULAR && TOPBuffer[x]>p2)
                 {
                 d1=Time[x];
                 p1=TOPBuffer[x];
                 break;
                 }   
              }
            }
         break;
        }
      }
      ObjectDelete("Dn_Trend");
      Draw_Trend("Dn_Trend",p1,d1,p2,d2,dn_color);
      
      double uptrend_val=NormalizeDouble(ObjectGetValueByShift("Up_Trend",1),Digits);
      double dntrend_val=NormalizeDouble(ObjectGetValueByShift("Dn_Trend",1),Digits);
      double Pre_uptrend_val=NormalizeDouble(ObjectGetValueByShift("Up_Trend",2),Digits);
      double Pre_dntrend_val=NormalizeDouble(ObjectGetValueByShift("Dn_Trend",2),Digits);
     
     // Up Trend break
     if(Close[2]>=Pre_uptrend_val && Close[1]<uptrend_val && CUR_BAR!=Bars)
     {
      if(Sound_Alert)
      Alert("Up Trend Line was broken at :"+TimeToStr(Time[1],TIME_DATE|TIME_SECONDS));
      if(Push_Alert)
      SendNotification("Up Trend Line was broken at :"+TimeToStr(Time[1],TIME_DATE|TIME_SECONDS));
      if(Email_Alert)
      SendMail("UP Trend Broken" ,"Up Trend Line was broken at :"+TimeToStr(Time[1],TIME_DATE|TIME_SECONDS));
      Print("Up Trend Line was broken at :"+TimeToStr(Time[1],TIME_DATE|TIME_SECONDS));
      CUR_BAR=Bars;
     }
     
     // Down Trend break
     if(Close[2]<=Pre_dntrend_val && Close[1]>dntrend_val && CUR_BAR!=Bars)
     {
      if(Sound_Alert)
      Alert("Down Trend Line was broken at :"+TimeToStr(Time[1],TIME_DATE|TIME_SECONDS));
      if(Push_Alert)
      SendNotification("Down Trend Line was broken at :"+TimeToStr(Time[1],TIME_DATE|TIME_SECONDS));
      if(Email_Alert)
      SendMail("Down Trend Broken" ,"Up Trend Line was broken at :"+TimeToStr(Time[1],TIME_DATE|TIME_SECONDS));
      Print("Down Trend Line was broken at :"+TimeToStr(Time[1],TIME_DATE|TIME_SECONDS));
      CUR_BAR=Bars;
     }
     
     CUR_BAR=Bars; 
//--- return value of prev_calculated for next call
   return(rates_total);
  }
//+------------------------------------------------------------------+
//------------------------------------------------------------------+
void Draw_Trend(string Name ,double p1 ,datetime d1 , double p2 , datetime d2,color CLR)
{
   if(ObjectCreate(ChartID(),Name,OBJ_TREND,0,d1,p1,d2,p2))
   {
    ObjectSetInteger(ChartID(),Name,OBJPROP_HIDDEN,true);
    ObjectSetInteger(ChartID(),Name,OBJPROP_RAY_RIGHT,true);
    ObjectSetInteger(ChartID(),Name,OBJPROP_COLOR,clrBlue);
    ObjectSetDouble(ChartID(),Name,OBJPROP_PRICE1,p1);
    ObjectSetDouble(ChartID(),Name,OBJPROP_PRICE2,p2);
    ObjectSetInteger(ChartID(),Name,OBJPROP_TIME1,d1);
    ObjectSetInteger(ChartID(),Name,OBJPROP_TIME2,d2);   
   ObjectSetInteger(ChartID(),Name,OBJPROP_COLOR,CLR);     
   }
}
